@@ -1,5 +1,7 @@
 import React, { PureComponent } from 'react';
 import uniqueId from 'lodash/uniqueId';
+import { Tooltip } from '../Tooltip/Tooltip';
+import * as PopperJS from 'popper.js';
 
 export interface Props {
   label: string;
@@ -7,12 +9,14 @@ export interface Props {
   className?: string;
   labelClass?: string;
   switchClass?: string;
+  tooltip?: string;
+  tooltipPlacement?: PopperJS.Placement;
   transparent?: boolean;
   onChange: (event?: React.SyntheticEvent<HTMLInputElement>) => void;
 }
 
 export interface State {
-  id: any;
+  id: string;
 }
 
 export class Switch extends PureComponent<Props, State> {
@@ -22,12 +26,20 @@ export class Switch extends PureComponent<Props, State> {
 
   internalOnChange = (event: React.FormEvent<HTMLInputElement>) => {
     event.stopPropagation();
-
     this.props.onChange(event);
   };
 
   render() {
-    const { labelClass = '', switchClass = '', label, checked, transparent, className } = this.props;
+    const {
+      labelClass = '',
+      switchClass = '',
+      label,
+      checked,
+      transparent,
+      className,
+      tooltip,
+      tooltipPlacement,
+    } = this.props;
 
     const labelId = this.state.id;
     const labelClassName = `gf-form-label ${labelClass} ${transparent ? 'gf-form-label--transparent' : ''} pointer`;
@@ -36,7 +48,18 @@ export class Switch extends PureComponent<Props, State> {
     return (
       <div className="gf-form-switch-container-react">
         <label htmlFor={labelId} className={`gf-form gf-form-switch-container ${className || ''}`}>
-          {label && <div className={labelClassName}>{label}</div>}
+          {label && (
+            <div className={labelClassName}>
+              {label}
+              {tooltip && (
+                <Tooltip placement={tooltipPlacement ? tooltipPlacement : 'auto'} content={tooltip} theme={'info'}>
+                  <div className="gf-form-help-icon gf-form-help-icon--right-normal">
+                    <i className="fa fa-info-circle" />
+                  </div>
+                </Tooltip>
+              )}
+            </div>
+          )}
           <div className={switchClassName}>
             <input id={labelId} type="checkbox" checked={checked} onChange={this.internalOnChange} />
             <span className="gf-form-switch__slider" />
